@@ -32,7 +32,7 @@ def get_datastore_by_ref(session, ds_ref):
 
     :param session: a vmware api session object
     :param ds_ref: managed object reference of a datastore
-    :rtype : a datastore object
+    :rtype: a datastore object
     """
     lst_properties = ["summary.type",
                       "summary.name",
@@ -113,7 +113,7 @@ class Datastore(object):
                             self.name)
 
     def __str__(self):
-        return '[%s]' % self._name
+        return '[%s]' % self.name
 
     def get_summary(self, session):
         """Get datastore summary.
@@ -142,7 +142,19 @@ class Datastore(object):
         for host_mount in host_mounts.DatastoreHostMount:
             if self.is_datastore_mount_usable(host_mount.mountInfo):
                 hosts.append(host_mount.key)
-        return hosts
+        connectables = []
+        if hosts:
+            host_runtimes = session.invoke_api(
+                vim_util,
+                'get_properties_for_a_collection_of_objects',
+                session.vim, 'HostSystem', hosts, ['runtime'])
+            for host_object in host_runtimes.objects:
+                host_props = vim_util.propset_dict(host_object.propSet)
+                host_runtime = host_props.get('runtime')
+                if hasattr(host_runtime, 'inMaintenanceMode') and (
+                        not host_runtime.inMaintenanceMode):
+                    connectables.append(host_object.obj)
+        return connectables
 
     @staticmethod
     def is_datastore_mount_usable(mount_info):
@@ -162,6 +174,9 @@ class Datastore(object):
 
     @staticmethod
     def choose_host(hosts):
+        if not hosts:
+            return None
+
         i = random.SystemRandom().randrange(0, len(hosts))
         return hosts[i]
 
@@ -180,6 +195,7 @@ class DatastorePath(object):
     file path to a virtual disk.
 
     Note:
+
     - Datastore path representations always uses forward slash as separator
       (hence the use of the posixpath module).
     - Datastore names are enclosed in square brackets.
