@@ -205,8 +205,8 @@ class VimUtilTest(base.TestCase):
             self.assertEqual([trav_spec], obj_spec.selectSet)
             self.assertFalse(obj_spec.skip)
 
-        vim.RetrievePropertiesEx.side_effect = \
-            vim_RetrievePropertiesEx_side_effect
+        vim.RetrievePropertiesEx.side_effect = (
+            vim_RetrievePropertiesEx_side_effect)
         vim_util.get_objects(vim, _type, max_objects)
         self.assertEqual(1, vim.RetrievePropertiesEx.call_count)
 
@@ -222,7 +222,8 @@ class VimUtilTest(base.TestCase):
         moref._type = "VirtualMachine"
         retrieve_result = mock.Mock()
 
-        def vim_RetrievePropertiesEx_side_effect(pc, specSet, options):
+        def vim_RetrievePropertiesEx_side_effect(pc, specSet, options,
+                                                 skip_op_id=False):
             self.assertTrue(pc is vim.service_content.propertyCollector)
             self.assertEqual(1, options.maxObjects)
 
@@ -245,8 +246,8 @@ class VimUtilTest(base.TestCase):
 
             return retrieve_result
 
-        vim.RetrievePropertiesEx.side_effect = \
-            vim_RetrievePropertiesEx_side_effect
+        vim.RetrievePropertiesEx.side_effect = (
+            vim_RetrievePropertiesEx_side_effect)
 
         res = vim_util.get_object_properties(vim, moref, None)
         self.assertEqual(1, vim.RetrievePropertiesEx.call_count)
@@ -316,6 +317,37 @@ class VimUtilTest(base.TestCase):
         vim.ContinueRetrievePropertiesEx.assert_called_once_with(
             vim.service_content.propertyCollector, token=token)
 
+    @mock.patch('oslo_vmware.vim_util.continue_retrieval')
+    @mock.patch('oslo_vmware.vim_util.cancel_retrieval')
+    def test_with_retrieval(self, cancel_retrieval, continue_retrieval):
+        vim = mock.Mock()
+        retrieve_result0 = mock.Mock()
+        retrieve_result0.objects = [mock.Mock(), mock.Mock()]
+        retrieve_result1 = mock.Mock()
+        retrieve_result1.objects = [mock.Mock(), mock.Mock()]
+        continue_retrieval.side_effect = [retrieve_result1, None]
+        expected = retrieve_result0.objects + retrieve_result1.objects
+
+        with vim_util.WithRetrieval(vim, retrieve_result0) as iterator:
+            self.assertEqual(expected, list(iterator))
+
+        calls = [
+            mock.call(vim, retrieve_result0),
+            mock.call(vim, retrieve_result1)]
+        continue_retrieval.assert_has_calls(calls)
+        self.assertFalse(cancel_retrieval.called)
+
+    @mock.patch('oslo_vmware.vim_util.continue_retrieval')
+    @mock.patch('oslo_vmware.vim_util.cancel_retrieval')
+    def test_with_retrieval_early_exit(self, cancel_retrieval,
+                                       continue_retrieval):
+        vim = mock.Mock()
+        retrieve_result = mock.Mock()
+        with vim_util.WithRetrieval(vim, retrieve_result):
+            pass
+
+        cancel_retrieval.assert_called_once_with(vim, retrieve_result)
+
     @mock.patch('oslo_vmware.vim_util.get_object_properties')
     def test_get_object_property(self, get_object_properties):
         prop = mock.Mock()
@@ -330,23 +362,23 @@ class VimUtilTest(base.TestCase):
         val = vim_util.get_object_property(vim, moref, property_name)
         self.assertEqual(prop.val, val)
         get_object_properties.assert_called_once_with(
-            vim, moref, [property_name])
+            vim, moref, [property_name], skip_op_id=False)
 
     def test_find_extension(self):
         vim = mock.Mock()
         ret = vim_util.find_extension(vim, 'fake-key')
         self.assertIsNotNone(ret)
         service_content = vim.service_content
-        vim.client.service.FindExtension.assert_called_once_with(
-            service_content.extensionManager, 'fake-key')
+        vim.FindExtension.assert_called_once_with(
+            service_content.extensionManager, extensionKey='fake-key')
 
     def test_register_extension(self):
         vim = mock.Mock()
         ret = vim_util.register_extension(vim, 'fake-key', 'fake-type')
         self.assertIsNone(ret)
         service_content = vim.service_content
-        vim.client.service.RegisterExtension.assert_called_once_with(
-            service_content.extensionManager, mock.ANY)
+        vim.RegisterExtension.assert_called_once_with(
+            service_content.extensionManager, extension=mock.ANY)
 
     def test_get_vc_version(self):
         session = mock.Mock()
@@ -398,3 +430,105 @@ class VimUtilTest(base.TestCase):
         entity = mock.Mock()
         inv_path = vim_util.get_inventory_path(session.vim, entity, 100)
         self.assertEqual('dc-1', inv_path)
+
+    def test_get_prop_spec(self):
+        client_factory = mock.Mock()
+        prop_spec = vim_util.get_prop_spec(
+            client_factory, "VirtualMachine", ["test_path"])
+        self.assertEqual(["test_path"], prop_spec.pathSet)
+        self.assertEqual("VirtualMachine", prop_spec.type)
+
+    def test_get_obj_spec(self):
+        client_factory = mock.Mock()
+        mock_obj = mock.Mock()
+        obj_spec = vim_util.get_obj_spec(
+            client_factory, mock_obj, select_set=["abc"])
+        self.assertEqual(mock_obj, obj_spec.obj)
+        self.assertFalse(obj_spec.skip)
+        self.assertEqual(["abc"], obj_spec.selectSet)
+
+    def test_get_prop_filter_spec(self):
+        client_factory = mock.Mock()
+        mock_obj = mock.Mock()
+        filter_spec = vim_util.get_prop_filter_spec(
+            client_factory, [mock_obj], ["test_prop"])
+        self.assertEqual([mock_obj], filter_spec.objectSet)
+        self.assertEqual(["test_prop"], filter_spec.propSet)
+
+    @mock.patch('oslo_vmware.vim_util.get_prop_spec')
+    @mock.patch('oslo_vmware.vim_util.get_obj_spec')
+    @mock.patch('oslo_vmware.vim_util.get_prop_filter_spec')
+    def _test_get_properties_for_a_collection_of_objects(
+            self, objs, max_objects,
+            mock_get_prop_filter_spec,
+            mock_get_obj_spec,
+            mock_get_prop_spec):
+        vim = mock.Mock()
+        if len(objs) == 0:
+            self.assertEqual(
+                [], vim_util.get_properties_for_a_collection_of_objects(
+                    vim, 'VirtualMachine', [], {}))
+            return
+
+        mock_prop_spec = mock.Mock()
+        mock_get_prop_spec.return_value = mock_prop_spec
+
+        mock_get_obj_spec.side_effect = [mock.Mock()
+                                         for obj in objs]
+        get_obj_spec_calls = [mock.call(vim.client.factory, obj)
+                              for obj in objs]
+
+        mock_prop_spec = mock.Mock()
+        mock_get_prop_spec.return_value = mock_prop_spec
+
+        mock_prop_filter_spec = mock.Mock()
+        mock_get_prop_filter_spec.return_value = mock_prop_filter_spec
+        mock_options = mock.Mock()
+        vim.client.factory.create.return_value = mock_options
+
+        mock_return_value = mock.Mock()
+        vim.RetrievePropertiesEx.return_value = mock_return_value
+        res = vim_util.get_properties_for_a_collection_of_objects(
+            vim, 'VirtualMachine', objs, ['runtime'], max_objects)
+        self.assertEqual(mock_return_value, res)
+
+        mock_get_prop_spec.assert_called_once_with(vim.client.factory,
+                                                   'VirtualMachine',
+                                                   ['runtime'])
+        self.assertEqual(get_obj_spec_calls, mock_get_obj_spec.mock_calls)
+        vim.client.factory.create.assert_called_once_with(
+            'ns0:RetrieveOptions')
+        self.assertEqual(max_objects if max_objects else len(objs),
+                         mock_options.maxObjects)
+        vim.RetrievePropertiesEx.assert_called_once_with(
+            vim.service_content.propertyCollector,
+            specSet=[mock_prop_filter_spec],
+            options=mock_options)
+
+    def test_get_properties_for_a_collection_of_objects(
+            self):
+        objects = ["m1", "m2"]
+        self._test_get_properties_for_a_collection_of_objects(objects, None)
+
+    def test_get_properties_for_a_collection_of_objects_max_objects_1(
+            self):
+        objects = ["m1", "m2"]
+        self._test_get_properties_for_a_collection_of_objects(objects, 1)
+
+    def test_get_properties_for_a_collection_of_objects_no_objects(
+            self):
+        self._test_get_properties_for_a_collection_of_objects([], None)
+
+    def test_propset_dict(self):
+        self.assertEqual({}, vim_util.propset_dict(None))
+
+        mock_propset = []
+        for i in range(2):
+            mock_obj = mock.Mock()
+            mock_obj.name = "test_name_%d" % i
+            mock_obj.val = "test_val_%d" % i
+            mock_propset.append(mock_obj)
+
+        self.assertEqual({"test_name_0": "test_val_0",
+                          "test_name_1": "test_val_1"},
+                         vim_util.propset_dict(mock_propset))
