@@ -22,9 +22,6 @@ import logging
 from oslo_utils import timeutils
 from suds import sudsobject
 
-from oslo_vmware._i18n import _LW
-
-
 LOG = logging.getLogger(__name__)
 
 
@@ -285,13 +282,14 @@ def get_objects(vim, type_, max_objects, properties_to_collect=None,
                                     options=options)
 
 
-def get_object_properties(vim, moref, properties_to_collect):
+def get_object_properties(vim, moref, properties_to_collect, skip_op_id=False):
     """Get properties of the given managed object.
 
     :param vim: Vim object
     :param moref: managed object reference
     :param properties_to_collect: names of the managed object properties to be
                                   collected
+    :param skip_op_id: whether to skip putting opID in the request
     :returns: properties of the given managed object
     :raises: VimException, VimFaultException, VimAttributeException,
              VimSessionOverLoadException, VimConnectionException
@@ -317,7 +315,8 @@ def get_object_properties(vim, moref, properties_to_collect):
     retrieve_result = vim.RetrievePropertiesEx(
         vim.service_content.propertyCollector,
         specSet=[property_filter_spec],
-        options=options)
+        options=options,
+        skip_op_id=skip_op_id)
     cancel_retrieval(vim, retrieve_result)
     return retrieve_result.objects
 
@@ -345,8 +344,8 @@ def get_object_properties_dict(vim, moref, properties_to_collect):
     # The object may have information useful for logging
     if hasattr(obj_contents[0], 'missingSet'):
         for m in obj_contents[0].missingSet:
-            LOG.warning(_LW("Unable to retrieve value for %(path)s "
-                            "Reason: %(reason)s"),
+            LOG.warning("Unable to retrieve value for %(path)s "
+                        "Reason: %(reason)s",
                         {'path': m.path,
                          'reason': m.fault.localizedMessage})
     return property_dict
@@ -389,17 +388,52 @@ def continue_retrieval(vim, retrieve_result):
         return vim.ContinueRetrievePropertiesEx(collector, token=token)
 
 
-def get_object_property(vim, moref, property_name):
+class WithRetrieval(object):
+    """Context to retrieve results.
+
+    This context provides an iterator to retrieve results and cancel (when
+    needed) retrieve operation on __exit__.
+
+    Example:
+
+      with WithRetrieval(vim, retrieve_result) as objects:
+          for obj in objects:
+              # Use obj
+    """
+
+    def __init__(self, vim, retrieve_result):
+        super(WithRetrieval, self).__init__()
+        self.vim = vim
+        self.retrieve_result = retrieve_result
+
+    def __enter__(self):
+        return iter(self)
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if self.retrieve_result:
+            cancel_retrieval(self.vim, self.retrieve_result)
+
+    def __iter__(self):
+        while self.retrieve_result:
+            for obj in self.retrieve_result.objects:
+                yield obj
+            self.retrieve_result = continue_retrieval(
+                self.vim, self.retrieve_result)
+
+
+def get_object_property(vim, moref, property_name, skip_op_id=False):
     """Get property of the given managed object.
 
     :param vim: Vim object
     :param moref: managed object reference
     :param property_name: name of the property to be retrieved
+    :param skip_op_id: whether to skip putting opID in the request
     :returns: property of the given managed object
     :raises: VimException, VimFaultException, VimAttributeException,
              VimSessionOverLoadException, VimConnectionException
     """
-    props = get_object_properties(vim, moref, [property_name])
+    props = get_object_properties(vim, moref, [property_name],
+                                  skip_op_id=skip_op_id)
     prop_val = None
     if props:
         prop = None
@@ -420,7 +454,7 @@ def find_extension(vim, key):
     :returns: the data object Extension or None
     """
     extension_manager = vim.service_content.extensionManager
-    return vim.client.service.FindExtension(extension_manager, key)
+    return vim.FindExtension(extension_manager, extensionKey=key)
 
 
 def register_extension(vim, key, type, label='OpenStack',
@@ -448,8 +482,8 @@ def register_extension(vim, key, type, label='OpenStack',
     desc.label = label
     desc.summary = summary
     os_ext.description = desc
-    os_ext.lastHeartbeatTime = timeutils.strtime()
-    vim.client.service.RegisterExtension(extension_manager, os_ext)
+    os_ext.lastHeartbeatTime = timeutils.utcnow().isoformat()
+    vim.RegisterExtension(extension_manager, extension=os_ext)
 
 
 def get_vc_version(session):
@@ -491,15 +525,14 @@ def get_inventory_path(vim, entity_ref, max_objects=100):
     entity_name = None
     propSet = None
     path = ""
-    while retrieve_result:
-        for obj in retrieve_result.objects:
+    with WithRetrieval(vim, retrieve_result) as objects:
+        for obj in objects:
             if hasattr(obj, 'propSet'):
                 propSet = obj.propSet
                 if len(propSet) >= 1 and not entity_name:
                     entity_name = propSet[0].val
                 elif len(propSet) >= 1:
                     path = '%s/%s' % (propSet[0].val, path)
-        retrieve_result = continue_retrieval(vim, retrieve_result)
     # NOTE(arnaud): slice to exclude the root folder from the result.
     if propSet is not None and len(propSet) > 0:
         path = path[len(propSet[0].val):]
@@ -520,3 +553,71 @@ def get_http_service_request_spec(client_factory, method, uri):
     http_service_request_spec.method = method
     http_service_request_spec.url = uri
     return http_service_request_spec
+
+
+def get_prop_spec(client_factory, spec_type, properties):
+    """Builds the Property Spec Object."""
+    prop_spec = client_factory.create('ns0:PropertySpec')
+    prop_spec.type = spec_type
+    prop_spec.pathSet = properties
+    return prop_spec
+
+
+def get_obj_spec(client_factory, obj, select_set=None):
+    """Builds the Object Spec object."""
+    obj_spec = client_factory.create('ns0:ObjectSpec')
+    obj_spec.obj = obj
+    obj_spec.skip = False
+    if select_set is not None:
+        obj_spec.selectSet = select_set
+    return obj_spec
+
+
+def get_prop_filter_spec(client_factory, obj_spec, prop_spec):
+    """Builds the Property Filter Spec Object."""
+    prop_filter_spec = client_factory.create('ns0:PropertyFilterSpec')
+    prop_filter_spec.propSet = prop_spec
+    prop_filter_spec.objectSet = obj_spec
+    return prop_filter_spec
+
+
+def get_properties_for_a_collection_of_objects(vim, type_,
+                                               obj_list, properties,
+                                               max_objects=None):
+    """Gets the list of properties for the collection of
+    objects of the type specified.
+    """
+    client_factory = vim.client.factory
+    if len(obj_list) == 0:
+        return []
+    prop_spec = get_prop_spec(client_factory, type_, properties)
+    lst_obj_specs = []
+    for obj in obj_list:
+        lst_obj_specs.append(get_obj_spec(client_factory, obj))
+    prop_filter_spec = get_prop_filter_spec(client_factory,
+                                            lst_obj_specs, [prop_spec])
+    options = client_factory.create('ns0:RetrieveOptions')
+    options.maxObjects = max_objects if max_objects else len(obj_list)
+    return vim.RetrievePropertiesEx(
+        vim.service_content.propertyCollector,
+        specSet=[prop_filter_spec], options=options)
+
+
+def propset_dict(propset):
+    """Turn a propset list into a dictionary
+
+    PropSet is an optional attribute on ObjectContent objects
+    that are returned by the VMware API.
+
+    You can read more about these at:
+    | http://pubs.vmware.com/vsphere-51/index.jsp
+    |    #com.vmware.wssdk.apiref.doc/
+    |        vmodl.query.PropertyCollector.ObjectContent.html
+
+    :param propset: a property "set" from ObjectContent
+    :return: dictionary representing property set
+    """
+    if propset is None:
+        return {}
+
+    return {prop.name: prop.val for prop in propset}
